Vehicle = 1; % APM 1.0 with ATmega 2560

% Generate data for ArduCopter
SENSOR_SIGN = [1, -1, -1,   -1, 1, 1];

%% Define variants/compiler flags
OUTPUTMODE = Simulink.Parameter;
OUTPUTMODE.Value = 1;
OUTPUTMODE.DataType = 'int16';
OUTPUTMODE.RTWInfo. StorageClass = 'Custom';
OUTPUTMODE.RTWInfo.CustomStorageClass = 'ImportedDefine';
OUTPUTMODE.RTWInfo.CustomAttributes.HeaderFile = 'ArduCopterDefines.h';

FLIGHT_MODE_X = mpt.Parameter;
FLIGHT_MODE_X.Value = 0;  %  0: '+' configuration; otherwise 'X' configuration
FLIGHT_MODE_X.DataType = 'int16';
FLIGHT_MODE_X.RTWInfo.StorageClass = 'Custom';
FLIGHT_MODE_X.RTWInfo.CustomStorageClass = 'ImportedDefine';
FLIGHT_MODE_X.RTWInfo.CustomAttributes.HeaderFile = 'ArduCopterDefines.h';

QUAD = mpt.Parameter;
QUAD.Value = 0;  % Quad vehicle
QUAD.DataType = 'int16';
QUAD.RTWInfo.StorageClass = 'Custom';
QUAD.RTWInfo.CustomStorageClass = 'ImportedDefine';
QUAD.RTWInfo.CustomAttributes.HeaderFile = 'ArduCopterDefines.h';

HEXA = mpt.Parameter;
HEXA.Value = 2;  % Hexa vehicle
HEXA.DataType = 'int16';
HEXA.RTWInfo.StorageClass = 'Custom';
HEXA.RTWInfo.CustomStorageClass = 'ImportedDefine';
HEXA.RTWInfo.CustomAttributes.HeaderFile = 'ArduCopterDefines.h';

AIRFRAME = mpt.Parameter;
AIRFRAME.Value = 0;
AIRFRAME.DataType = 'int16';
AIRFRAME.RTWInfo.StorageClass = 'Custom';
AIRFRAME.RTWInfo.CustomStorageClass = 'ImportedDefine';
AIRFRAME.RTWInfo.CustomAttributes.HeaderFile = 'ArduCopterDefines.h';

UseBMP = mpt.Parameter;
UseBMP.Value = 0;
UseBMP.DataType = 'int16';
UseBMP.RTWInfo.StorageClass = 'Custom';
UseBMP.RTWInfo.CustomStorageClass = 'ImportedDefine';
UseBMP.RTWInfo.CustomAttributes.HeaderFile = 'ArduCopterDefines.h';

IsSONAR = mpt.Parameter;
IsSONAR.Value = 0;
IsSONAR.DataType = 'int16';
IsSONAR.RTWInfo.StorageClass = 'Custom';
IsSONAR.RTWInfo.CustomStorageClass = 'ImportedDefine';
IsSONAR.RTWInfo.CustomAttributes.HeaderFile = 'ArduCopterDefines.h';

IsGPS = mpt.Parameter;
IsGPS.Value = 0;
IsGPS.DataType = 'int16';
IsGPS.RTWInfo.StorageClass = 'Custom';
IsGPS.RTWInfo.CustomStorageClass = 'ImportedDefine';
IsGPS.RTWInfo.CustomAttributes.HeaderFile = 'ArduCopterDefines.h';

BATTERY_EVENT = mpt.Parameter;
BATTERY_EVENT.Value = 0;
BATTERY_EVENT.DataType = 'int16';
BATTERY_EVENT.RTWInfo.StorageClass = 'Custom';
BATTERY_EVENT.RTWInfo.CustomStorageClass = 'ImportedDefine';
BATTERY_EVENT.RTWInfo.CustomAttributes.HeaderFile = 'ArduCopterDefines.h';

%% Define Variants
AccelOnlyAngles = Simulink.Variant;
AccelOnlyAngles.Condition = 'OUTPUTMODE == 2';
DCMBasedAngles = Simulink.Variant;
DCMBasedAngles.Condition = 'OUTPUTMODE == 0 || OUTPUTMODE == 1';
CorrectedMode = Simulink.Variant;
CorrectedMode.Condition = 'OUTPUTMODE == 1';
UncorrectedMode = Simulink.Variant;
UncorrectedMode.Condition = 'OUTPUTMODE == 0';

QUAD_Var = Simulink.Variant;
QUAD_Var.Condition = 'AIRFRAME == QUAD';

HEXA_Var = Simulink.Variant;
HEXA_Var.Condition = 'AIRFRAME == HEXA';

FLIGHT_MODE_X_Var = Simulink.Variant;
FLIGHT_MODE_X_Var.Condition = 'FLIGHT_MODE_X~=0';

FLIGHT_MODE_Plus_Var = Simulink.Variant;
FLIGHT_MODE_Plus_Var.Condition = 'FLIGHT_MODE_X==0';

%%
SW_DIP1 = 0; % flight orientation 0: '+' configuration, 1: 'x' configuration
SW_DIP3 = 0; % flight mode
%flightMode definitions
%Modes
FM_ACRO_MODE          = 0;  % DIP3 down (ON)  = Acrobatic Mode
FM_STABLE_MODE        = 1;  % DIP3 up   (OFF) = Stable Mode.
AP_NORMAL_STABLE_MODE = 2;  % Just Stable Mode 
AP_ALTITUDE_HOLD      = 3;  % Just Altitude Hold
AP_GPS_HOLD           = 4;  % Just GPS Hold
AP_ALT_GPS_HOLD       = 5;  % Full Automatic (GPS and Altitude Hold)
%AP_WAYPOINT         =   6;  % Waypoint Navigation...NOT USED YET

ALTITUDE_CONTROL_NONE = 0;
ALTITUDE_CONTROL_SONAR = 1;


GPS = 0;

Gyro_Gain_X = 0.4;% in degrees
Gyro_Gain_Y = 0.4;% in degrees
Gyro_Gain_Z = 0.4;% in degrees

GRAVITY = int16(408);
MAX_CONTROL_OUTPUT = 250;
MIN_CONTROL_OUTPUT = -250;
AP_AUTOMATIC_MODE = 1;
AP_NORMAL_MODE = 0;
STABLE_MODE = 0;
ACRO_MODE = 1;
STICK_TO_ANGLE_FACTOR = 12.0;
YAW_STICK_TO_ANGLE_FACTOR = 150.0;

DISARM_DELAY = 25; % 0.5 seconds
ARM_DELAY = 50; % 2 seconds - RC updates at 50 Hz
SAFETY_MAX_THROTTLE_INCREASE = 100;
SAFETY_HOVER_THROTTLE = 1300;
SAFETY_DELAY = 25; % 0.5 seconds

cellInfo = { ... 
  { ... 
    'EEPROM_Bus', ... 
    '', ... 
    sprintf(''), ... 
    'Auto', ... 
    '-1', {... 
      {'KP_QUAD_ROLL',              1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'STABLE_MODE_KP_RATE_ROLL',  1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KI_QUAD_ROLL',              1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KP_QUAD_PITCH',             1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'STABLE_MODE_KP_RATE_PITCH', 1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KI_QUAD_PITCH',             1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KP_QUAD_YAW',               1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'STABLE_MODE_KP_RATE_YAW'    1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KI_QUAD_YAW'                1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'STABLE_MODE_KP_RATE'        1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KP_GPS_ROLL'                1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KD_GPS_ROLL'                1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KI_GPS_ROLL'                1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KP_GPS_PITCH'               1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KD_GPS_PITCH'               1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KI_GPS_PITCH'               1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'GPS_MAX_ANGLE'              1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KP_ALTITUDE'                1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KD_ALTITUDE'                1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KI_ALTITUDE'                1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'ACC_OFFSET_X'               1, 'int16',  -1, 'real', 'Sample', 'Fixed', [], []}; ...% Integer value
      {'ACC_OFFSET_Y'               1, 'int16',  -1, 'real', 'Sample', 'Fixed', [], []}; ...% Integer value
      {'ACC_OFFSET_Z'               1, 'int16',  -1, 'real', 'Sample', 'Fixed', [], []}; ...% Integer value
      {'GYRO_OFFSET_ROLL'           1, 'int16',  -1, 'real', 'Sample', 'Fixed', [], []}; ...% Integer value
      {'GYRO_OFFSET_PITCH'          1, 'int16',  -1, 'real', 'Sample', 'Fixed', [], []}; ...% Integer value
      {'GYRO_OFFSET_YAW'            1, 'int16',  -1, 'real', 'Sample', 'Fixed', [], []}; ...% Integer value
      {'KP_ROLLPITCH'               1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KI_ROLLPITCH'               1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KP_YAW'                     1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KI_YAW'                     1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'GEOG_CORRECTION_FACTOR'     1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'MAGNETOMETER'               1, 'int16',  -1, 'real', 'Sample', 'Fixed', [], []}; ...% Integer value
      {'XMITFACTOR'                 1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KP_RATEROLL'                1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KI_RATEROLL'                1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KD_RATEROLL'                1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KP_RATEPITCH'               1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KI_RATEPITCH'               1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KD_RATEPITCH'               1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KP_RATEYAW'                 1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KI_RATEYAW'                 1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KD_RATEYAW'                 1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'ROLL_MID'                   1, 'int16',  -1, 'real', 'Sample', 'Fixed', [], []}; ...% Integer value
      {'PITCH_MID'                  1, 'int16',  -1, 'real', 'Sample', 'Fixed', [], []}; ...% Integer value
      {'YAW_MID'                    1, 'int16',  -1, 'real', 'Sample', 'Fixed', [], []}; ...% Integer value
      {'CH_ROLL_SLOPE'              1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'CH_PITCH_SLOPE'             1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'CH_THROTTLE_SLOPE'          1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'CH_YAW_SLOPE'               1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'CH_AUX_SLOPE'               1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'CH_AUX2_SLOPE'              1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'CH_ROLL_OFFSET'             1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'CH_PITCH_OFFSET'            1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'CH_THROTTLE_OFFSET'         1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'CH_YAW_OFFSET'              1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'CH_AUX_OFFSET'              1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'CH_AUX2_OFFSET'             1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'CAM_MODE'                   1, 'uint8',  -1, 'real', 'Sample', 'Fixed', [], []}; ...% Byte value
      {'MAG_ORIENTATION'            1, 'uint8',  -1, 'real', 'Sample', 'Fixed', [], []}; ...% Byte value
      {'MAG_DECLINATION'            1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'MAG_OFFSET_X'               1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'MAG_OFFSET_Y'               1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'MAG_OFFSET_Z'               1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'MIN_THROTTLE'               1, 'int16',  -1, 'real', 'Sample', 'Fixed', [], []}; ...% Integer value
      {'KP_RF_ROLL'                 1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KI_RF_ROLL'                 1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KD_RF_ROLL'                 1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KP_RF_PITCH'                1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KI_RF_PITCH'                1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KD_RF_PITCH'                1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'RF_MAX_ANGLE'               1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'RF_SAFETY_ZONE'             1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KP_SONAR_ALTITUDE'          1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KI_SONAR_ALTITUDE'          1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
      {'KD_SONAR_ALTITUDE'          1, 'single', -1, 'real', 'Sample', 'Fixed', [], []}; ...
    } ...
  } ...
}'; 
Simulink.Bus.cellToObject(cellInfo);

% Create initialization structure for bus
GainScaleFactor = single(0.75);
EEPROM_Struct.KP_QUAD_ROLL               = single(4.0*GainScaleFactor); % Address 0
EEPROM_Struct.STABLE_MODE_KP_RATE_ROLL   = single(1.2*GainScaleFactor); % Address 4
EEPROM_Struct.KI_QUAD_ROLL               = single(0.15*GainScaleFactor); % Address 8
EEPROM_Struct.KP_QUAD_PITCH              = single(4.0*GainScaleFactor); % Address 12
EEPROM_Struct.STABLE_MODE_KP_RATE_PITCH  = single(1.2*GainScaleFactor); % Address 16
EEPROM_Struct.KI_QUAD_PITCH              = single(0.15*GainScaleFactor); % Address 20
EEPROM_Struct.KP_QUAD_YAW                = single(3.0); % Address 24
EEPROM_Struct.STABLE_MODE_KP_RATE_YAW    = single(2.4); % Address 28
EEPROM_Struct.KI_QUAD_YAW                = single(0.15); % Address 32
EEPROM_Struct.STABLE_MODE_KP_RATE        = single(0.2); % Address 36
EEPROM_Struct.KP_GPS_ROLL                = single(0.012); % Address 40
EEPROM_Struct.KD_GPS_ROLL                = single(0.015); % Address 44
EEPROM_Struct.KI_GPS_ROLL                = single(0.001); % Address 48
EEPROM_Struct.KP_GPS_PITCH               = single(0.010); % Address 52
EEPROM_Struct.KD_GPS_PITCH               = single(0.015); % Address 56
EEPROM_Struct.KI_GPS_PITCH               = single(0.001); % Address 60
EEPROM_Struct.GPS_MAX_ANGLE              = single(22.0); % Address 64
EEPROM_Struct.KP_ALTITUDE                = single(0.08); % Address 68
EEPROM_Struct.KD_ALTITUDE                = single(0.06); % Address 72
EEPROM_Struct.KI_ALTITUDE                = single(0.05); % Address 76

% Define the offsets based on measurements from the test vehicles
if (Vehicle == 0) % First test vehicle - ATmega1280
    EEPROM_Struct.ACC_OFFSET_X               = int16(2047); %= int16(2062); % Address 80
    EEPROM_Struct.ACC_OFFSET_Y               = int16(2027); %= int16(2009); % Address 84
    EEPROM_Struct.ACC_OFFSET_Z               = int16(2481-408); %= int16(2070); % Address 88     % Add gravity component
    EEPROM_Struct.GYRO_OFFSET_ROLL           = int16(1603); %= int16(1659); % Address 92
    EEPROM_Struct.GYRO_OFFSET_PITCH          = int16(1623); %= int16(1650); % Address 96
    EEPROM_Struct.GYRO_OFFSET_YAW            = int16(1699); %= int16(1650); % Address 100
elseif (Vehicle == 1) % Second test vehicle - ATMega2560
    EEPROM_Struct.ACC_OFFSET_X               = int16(2018); %= int16(2062); % Address 80
    EEPROM_Struct.ACC_OFFSET_Y               = int16(2036); %= int16(2009); % Address 84
    EEPROM_Struct.ACC_OFFSET_Z               = int16(2496-408); %= int16(2070); % Address 88     % Add gravity component
    EEPROM_Struct.GYRO_OFFSET_ROLL           = int16(1716); %= int16(1659); % Address 92
    EEPROM_Struct.GYRO_OFFSET_PITCH          = int16(1588); %= int16(1650); % Address 96
    EEPROM_Struct.GYRO_OFFSET_YAW            = int16(1679); %= int16(1650); % Address 100
end

EEPROM_Struct.KP_ROLLPITCH               = single(0.0014); % Address 104
EEPROM_Struct.KI_ROLLPITCH               = single(0.00000015); % Address 108
EEPROM_Struct.KP_YAW                     = single(1.0); % Address 112
EEPROM_Struct.KI_YAW                     = single(0.00002); % Address 116
EEPROM_Struct.GEOG_CORRECTION_FACTOR     = single(0.0); % Address 120
EEPROM_Struct.MAGNETOMETER               = int16(0); % Address 124
EEPROM_Struct.XMITFACTOR                 = single(0.32); % Address 128
EEPROM_Struct.KP_RATEROLL                = single(1.95); % Address 132
EEPROM_Struct.KI_RATEROLL                = single(0.0); % Address 136
EEPROM_Struct.KD_RATEROLL                = single(0.0); % Address 140
EEPROM_Struct.KP_RATEPITCH               = single(1.95); % Address 144
EEPROM_Struct.KI_RATEPITCH               = single(0.0); % Address 148
EEPROM_Struct.KD_RATEPITCH               = single(0.0); % Address 152
EEPROM_Struct.KP_RATEYAW                 = single(3.2); % Address 156
EEPROM_Struct.KI_RATEYAW                 = single(0.0); % Address 160
EEPROM_Struct.KD_RATEYAW                 = single(0.0); % Address 164
EEPROM_Struct.ROLL_MID                   = int16(1500); % Address 168
EEPROM_Struct.PITCH_MID                  = int16(1500); % Address 172
EEPROM_Struct.YAW_MID                    = int16(1500); % Address 176
EEPROM_Struct.CH_ROLL_SLOPE              = single(1.0); %= single(1.27); % Address 180
EEPROM_Struct.CH_PITCH_SLOPE             = single(1.0); %= single(1.25); % Address 184
EEPROM_Struct.CH_THROTTLE_SLOPE          = single(1.0); %= single(1.29); % Address 188
EEPROM_Struct.CH_YAW_SLOPE               = single(1.0); %= single(1.28); % Address 192
EEPROM_Struct.CH_AUX_SLOPE               = single(1.0); %= single(1.28); % Address 196
EEPROM_Struct.CH_AUX2_SLOPE              = single(1.0); %= single(1.78); % Address 200
EEPROM_Struct.CH_ROLL_OFFSET             = single(0.0); %= single(-356.2); % Address 204
EEPROM_Struct.CH_PITCH_OFFSET            = single(0.0); %= single(-322.9); % Address 208
EEPROM_Struct.CH_THROTTLE_OFFSET         = single(0.0); %= single(-380.5); % Address 212
EEPROM_Struct.CH_YAW_OFFSET              = single(0.0); %= single(-382.5); % Address 216
EEPROM_Struct.CH_AUX_OFFSET              = single(0.0); %= single(-369.1); % Address 220
EEPROM_Struct.CH_AUX2_OFFSET             = single(0.0); %= single(-604.3); % Address 224
EEPROM_Struct.CAM_MODE                   = uint8(0); % Address 228
EEPROM_Struct.MAG_ORIENTATION            = uint8(0); % Address 232
EEPROM_Struct.MAG_DECLINATION            = single(0.0); % Address 236
EEPROM_Struct.MAG_OFFSET_X               = single(-90.5); % Address 240
EEPROM_Struct.MAG_OFFSET_Y               = single(-133); % Address 244
EEPROM_Struct.MAG_OFFSET_Z               = single(-133.5); % Address 248
EEPROM_Struct.MIN_THROTTLE               = int16(1052); % Address 252
EEPROM_Struct.KP_RF_ROLL                 = single(0.10); % Address 256
EEPROM_Struct.KI_RF_ROLL                 = single(0.00); % Address 260
EEPROM_Struct.KD_RF_ROLL                 = single(0.03); % Address 264
EEPROM_Struct.KP_RF_PITCH                = single(0.10); % Address 268
EEPROM_Struct.KI_RF_PITCH                = single(0.00); % Address 272
EEPROM_Struct.KD_RF_PITCH                = single(0.03); % Address 276
EEPROM_Struct.RF_MAX_ANGLE               = single(10.0); % Address 280
EEPROM_Struct.RF_SAFETY_ZONE             = single(120.0); % Address 284
EEPROM_Struct.KP_SONAR_ALTITUDE          = single(0.8); % Address 288
EEPROM_Struct.KI_SONAR_ALTITUDE          = single(0.3); % Address 292
EEPROM_Struct.KD_SONAR_ALTITUDE          = single(0.7); % Address 296

EEPROM_Parameter = mpt.Parameter;
EEPROM_Parameter.Value = EEPROM_Struct;
EEPROM_Parameter.DataType = 'Bus: EEPROM_Bus';
