%% TX calibrate
% The objective is to get the joystick interface to reproduce the
% measured JR 6102 outputs that are captured using:
% RC_capture.mdl and RC_Display_2.mdl files
% Y=A*x+B   x=output from USB Joystick interface

% Note that the TX reversing function are set as follows:
%  THR N        AIL  R
%  ELE R        RUD  R
%  GEA N        FLA  N
%  N=normal  R=reversed

if ~(exist('controller','var'))
    disp('controller not specified, assuming typical joystick');
    controller = 3;
end

%% Dick Benson JR6102
if (controller == 1)
% Measured using TX and RX on copter with RC_Capture.mdl and
% RC_Display_2.mdl
    Roll_Left  = 1058;  % stick left
    Roll_Right = 1850;  % stick right
    Pitch_Down = 1066;  % stick forward
    Pitch_Up   = 1856;  % stick back
    Yaw_Left   = 1060;  % stick left
    Yaw_Right  = 1852;  % stick right
    Throttle_Max = 1874; % stick forward
    Throttle_Min = 1080; % stick aft
    Left_SW_Back = 1854; % switch towards back of TX
    Left_SW_Fwd  = 1072;
    Right_SW_Back= 1582; % switch towards back of TX
    Right_SW_Fwd  = 1194;
    
    % Measured with JR_6102.mdl using JR to USB adaptor GWFSM002
    mr_left=-.5313; mr_right=.7381; % Roll
    mp_down=.7656; mp_up=.131;      % Pitch
    my_left=-.5391; my_right=.7381; % Yaw
    mt_min=-.513; mt_max=.7461;     % Throttle
    mls_back=.131; mls_fwd=.757;    % Aux (Left Switch)
    mrs_back=.353; mrs_fwd=.664;    % Aux2 (Right Switch)
    
%% Richard Ruff JR6102
elseif (controller == 2)
    % Measured using TX and RX on copter with RC_Capture.mdl and
    % RC_Display_2.mdl
    % Duplicate of Dick's for simulation pending actual hardware
    Roll_Left  = 1058;   % stick left
    Roll_Right = 1850;   % stick right
    Pitch_Down = 1066;   % stick forward
    Pitch_Up   = 1856;   % stick back
    Yaw_Left   = 1060;   % stick left
    Yaw_Right  = 1852;   % stick right
    Throttle_Max = 1874; % stick forward
    Throttle_Min = 1080; % stick aft
    Left_SW_Back = 1854; % switch towards back of TX
    Left_SW_Fwd  = 1072;
    Right_SW_Back= 1582; % switch towards back of TX
    Right_SW_Fwd  = 1194;
    
    % Measured with JR_6102.mdl using JR to USB adaptor GWFSM002B
    mr_left=-.6484; mr_right=.6111; % Roll
    mp_down=.8242; mp_up=.1944;     % Pitch up is stick down, pitch down is stick forward
    my_left=-.6484; my_right=.6111; % Yaw
    mt_min=-.625; mt_max=.6032;     % Throttle
    mls_back=.1944; mls_fwd=.8203;  % Aux (Left Switch)
    mrs_back=.5195; mrs_fwd=.832;   % Aux2 (Right Switch)
    
%% Logitech Extreme 3D Pro Joystick
elseif (controller == 3)
    % Measured using TX and RX on copter with RC_Capture.mdl and
    % RC_Display_2.mdl
    % Duplicate of Dick's for simulation pending actual hardware
    Roll_Left  = 1058;   % stick left
    Roll_Right = 1850;   % stick right
    Pitch_Down = 1066;   % stick forward
    Pitch_Up   = 1856;   % stick back
    Yaw_Left   = 1060;   % stick left
    Yaw_Right  = 1852;   % stick right
    Throttle_Max = 1874; % stick forward
    Throttle_Min = 1080; % stick aft
    Left_SW_Back = 1854; % switch towards back of TX
    Left_SW_Fwd  = 1072;
    Right_SW_Back= 1582; % switch towards back of TX
    Right_SW_Fwd  = 1194;
    
    % Measured with JR_6102.mdl using JR to USB adaptor GWFSM002B
    mr_left=-1; mr_right=1; % Roll
    mp_down=-1; mp_up=1;     % Pitch
    my_left=-1; my_right=1; % Yaw
    mt_min=0; mt_max=1;     % Throttle
    mls_back=0; mls_fwd=1;  % Aux (Left Switch)
    mrs_back=0; mrs_fwd=1;  % Aux2 (Right Switch)
    
%% Signal Builder with predefined flight profiles
% elseif (controller == 4)
% TBD
%% 3D Connexion Space Navigator
elseif (controller == 5)
    % Measured using TX and RX on copter with RC_Capture.mdl and
    % RC_Display_2.mdl
    % Duplicate of Dick's for simulation pending actual hardware
    Roll_Left  = 1000;   % stick left
    Roll_Right = 2000;   % stick right
    Pitch_Down = 1000;   % stick forward
    Pitch_Up   = 2000;   % stick back
    Yaw_Left   = 1000;   % stick left
    Yaw_Right  = 2000;   % stick right
    Throttle_Max = 2000; % stick forward
    Throttle_Min = 1000; % stick aft
    Left_SW_Back = 2000; % switch towards back of TX
    Left_SW_Fwd  = 1000;
    Right_SW_Back= 2000; % switch towards back of TX
    Right_SW_Fwd  = 1000;
    
    % Measured with JR_6102.mdl using JR to USB adaptor GWFSM002B
    mr_left=-1; mr_right=1; % Roll
    mp_down=-1; mp_up=1;     % Pitch
    my_left=-1; my_right=1; % Yaw
    mt_min=0; mt_max=1;     % Throttle
    mls_back=0; mls_fwd=1;  % Aux (Left Switch)
    mrs_back=0; mrs_fwd=1;  % Aux2 (Right Switch)
    
else
    disp('No input device specified.');
% Default values - may need to be tweaked in future.

    Roll_Left  = 1058;   % stick left
    Roll_Right = 1850;   % stick right
    Pitch_Down = 1066;   % stick forward
    Pitch_Up   = 1856;   % stick back
    Yaw_Left   = 1060;   % stick left
    Yaw_Right  = 1852;   % stick right
    Throttle_Max = 1874; % stick forward
    Throttle_Min = 1080; % stick aft
    Left_SW_Back = 1854; % switch towards back of TX
    Left_SW_Fwd  = 1072;
    Right_SW_Back= 1582; % switch towards back of TX
    Right_SW_Fwd  = 1194;

    mr_left=-1; mr_right=1; % Roll
    mp_down=-1; mp_up=1;    % Pitch
    my_left=-1; my_right=1; % Yaw
    mt_min=0;   mt_max=1;   % Throttle
    mls_back=0; mls_fwd=1;  % Aux (Left Switch)
    mrs_back=0; mrs_fwd=1;  % Aux2 (Right Switch)
    
end

%% Calculate bias and scaling
TX.A_Roll= (Roll_Right-Roll_Left)/(mr_right-mr_left);
TX.B_Roll= Roll_Left-TX.A_Roll*mr_left;

TX.A_Pitch= (Pitch_Up-Pitch_Down)/(mp_up-mp_down);
TX.B_Pitch= Pitch_Up-TX.A_Pitch*mp_up;

TX.A_Yaw=(Yaw_Right-Yaw_Left)/(my_right-my_left);
TX.B_Yaw= Yaw_Left-TX.A_Yaw*my_left;

TX.A_Thr=(Throttle_Max-Throttle_Min)/(mt_max-mt_min);
TX.B_Thr=Throttle_Min-TX.A_Thr*mt_min;

TX.A_Lsw = (Left_SW_Back-Left_SW_Fwd )/(mls_back-mls_fwd);
TX.B_Lsw = Left_SW_Back-TX.A_Lsw*mls_back;

TX.A_Rsw = (Right_SW_Back-Right_SW_Fwd)/(mrs_back-mrs_fwd);
TX.B_Rsw = Right_SW_Back-TX.A_Rsw*mrs_back;





