%% Arduino Setup script
ProjDir = pwd;

disp('Set up to use hardware? y/n [n]?');
disp('This will connect to the target board and requires the Arduino IDE be installed');
reply = input('Answer ''n'' to use simulation mode only.', 's');

if (~isempty(reply) && strncmpi(reply,'y',1))
    disp('Setting Arduino preferences');
    disp('');
    disp('');
    disp('Specify the Arduino IDE location');
    disp('Note: The Hover Demo was built using the arduino-022 IDE');
    ArduinoDir = uigetdir;
    arduino.Prefs.setArduinoPath(ArduinoDir);
    arduino.Prefs.setBoard('mega');
    try
        comPorts=arduino.Prefs.searchForComPort;
        arduino.Prefs.setComPort(comPorts{1});
    catch
        disp('Failed to find/set com port for Arduino board');
        disp('Check if Arduino board is connected to computer');
        disp('This step can be done by running ArduinoSetup located in the Setup folder');
    end
end
%% Rebuild S-functions for local machine
disp('Building Arduino Library blocks');

cd ([ProjDir,'\Arduino\blocks\ADC_IMU']);
mex Arduino_ADCRead_sfcn.c

cd ([ProjDir,'\Arduino\blocks\Analog']);
mex sfunar_analogInput.c
mex sfunar_analogOutput.c

cd ([ProjDir,'\Arduino\blocks\Digital']);
mex sfunar_digitalInput.c
mex sfunar_digitalOutput.c

cd ([ProjDir,'\Arduino\blocks\EEPROM']);
mex Arduino_EEPROMRead_sfcn.c

cd ([ProjDir,'\Arduino\blocks\LCD']);
mex sfunar_lcdOutput.c

cd ([ProjDir,'\Arduino\blocks\RCChannels']);
mex Arduino_RCWrite_sfcn.c
mex Arduino_RCRead_sfcn.c

cd ([ProjDir,'\Arduino\blocks\Serial']);
mex sfunar_serialConfig.c
mex sfunar_serialRead.c
mex sfunar_serialWrite.c

% cd ([ProjDir,'\Arduino\blocks\Serial3']);
% mex sfunar_serial3Config.c
% mex sfunar_serial3Read.c
% mex sfunar_serial3Write.c

cd ([ProjDir,'\Arduino\blocks\Servo']);
mex sfunar_servoOutput.c

%% Complete setup
cd (ProjDir);
disp('Arduino Setup complete');